[PATCH] anti-cross-CPU printk/oops interleaving

From: David Howells (dhowells@redhat.com)
Date: Fri Jul 04 2003 - 04:11:18 EST


Hi Linus,

This patch prevents an oops on one CPU being interleaved char-by-char with
printks being performed on other CPUs, thus rendering them actually readable
under those circumstances.

David

diff -ur linux-2.5.74/kernel/printk.c linux-2.5.74-auto/kernel/printk.c
--- linux-2.5.74/kernel/printk.c 2003-07-03 15:37:24.000000000 +0100
+++ linux-2.5.74-auto/kernel/printk.c 2003-07-03 15:41:15.000000000 +0100
@@ -378,6 +378,9 @@
                 logged_chars++;
 }
 
+/* cpu currently holding logbuf_lock */
+static volatile int printk_cpu = -1;
+
 /*
  * This is printk. It can be called from any context. We want it to work.
  *
@@ -400,8 +403,9 @@
         static char printk_buf[1024];
         static int log_level_unknown = 1;
 
- if (oops_in_progress) {
- /* If a crash is occurring, make sure we can't deadlock */
+ if (oops_in_progress && printk_cpu == smp_processor_id()) {
+ /* If a crash is occurring during printk() on this CPU,
+ * make sure we can't deadlock */
                 spin_lock_init(&logbuf_lock);
                 /* And make sure that we print immediately */
                 init_MUTEX(&console_sem);
@@ -409,6 +413,7 @@
 
         /* This stops the holder of console_sem just where we want him */
         spin_lock_irqsave(&logbuf_lock, flags);
+ printk_cpu = smp_processor_id();
 
         /* Emit the output into the temporary buffer */
         va_start(args, fmt);
-
To unsubscribe from this list: send the line "unsubscribe linux-kernel" in
the body of a message to majordomo@vger.kernel.org
More majordomo info at http://vger.kernel.org/majordomo-info.html
Please read the FAQ at http://www.tux.org/lkml/



This archive was generated by hypermail 2b29 : Mon Jul 07 2003 - 22:00:22 EST